"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const base_1 = require("../base");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const logger_1 = require("../services/logger");
const render_1 = require("../services/render");
const validate_1 = require("../services/validate");
class Publish extends base_1.default {
    async preInit() {
        const { flags } = this.parse(Publish);
        this.tag = flags.tag;
        if (this.tag === 'latest') {
            throw Error('rde will automatic add latest for you, please use semver-like tag');
        }
        this.rdcConf = await validate_1.validateRdc(true);
        return flags;
    }
    async initialize() {
        const { docker: dockerConf } = this.rdcConf;
        this.rdc = dockerConf.tag;
        const name = this.rdc.split(':')[0];
        await docker_1.default.genDockerFile4Publish(`${conf_1.default.dockerWorkDirRoot}/${name}`, this.rdcConf.extends || 'node:latest', conf_1.default.localCacheDir);
    }
    async run() {
        const name = this.rdc.split(':')[0];
        await docker_1.default.build(this.rdc, conf_1.default.localCacheDir, true, '../');
        await docker_1.default.tag(this.rdc, `${name}:${this.tag}`);
        await docker_1.default.tag(this.rdc, `${name}:latest`);
        await docker_1.default.push(name);
        this.tag = `${name}:${this.tag}`;
    }
    async postRun() {
        const { docker: dockerConf } = this.rdcConf;
        dockerConf.tag = this.tag;
        await render_1.default.renderTo('module', {
            obj: this.rdcConf,
        }, conf_1.default.rdcConfName, { overwrite: true });
        logger_1.logger.star(`Published ${this.tag} successfully`);
    }
}
Publish.examples = [
    '$ rde publish',
];
Publish.flags = Object.assign({}, base_1.default.flags, { tag: command_1.flags.string({
        char: 't',
        description: 'as your image tag for pushing to docker hub',
        required: true,
    }) });
exports.default = Publish;
