"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const signale = require("signale");
signale.config({
    displayBadge: true,
    displayTimestamp: true,
});
const { Signale } = signale;
exports.logger = signale.scope('RDE');
class Spinner {
    constructor() {
        this.interval = 80;
        this.frames = [
            ['🌑 ', '🌒 ', '🌓 ', '🌔 ', '🌕 ', '🌖 ', '🌗 ', '🌘 '],
            ['🙈 ', '🙈 ', '🙉 ', '🙊 '],
            ['.  ', '.. ', '...', ' ..', '  .', '   '],
            ['[    ]', '[=   ]', '[==  ]', '[=== ]', '[ ===]', '[  ==]', '[   =]', '[    ]',
                '[   =]', '[  ==]', '[ ===]', '[====]', '[=== ]', '[==  ]', '[=   ]'],
            ['🌲', '🎄'],
            ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏'],
        ];
        this.logger = null;
        this.timerId = null;
        this.counter = 0;
    }
    start(info) {
        if (!this.logger) {
            this.logger = new Signale({ interactive: true, scope: 'RDE' });
            this.logger.config({
                displayBadge: true,
                displayTimestamp: true,
            });
        }
        if (this.timerId) {
            clearInterval(this.timerId);
        }
        const index = Math.floor(Math.random() * this.frames.length);
        const frames = this.frames[index];
        this.timerId = setInterval(() => {
            this.counter = this.counter % (frames.length);
            this.logger.await(`%s: ${info}`, frames[this.counter]);
            this.counter++;
        }, this.interval);
    }
    stop() {
        clearInterval(this.timerId);
        this.logger = null;
    }
}
exports.spinner = new Spinner();
// only for async function
function log(message) {
    return (_t, _p, descriptor) => {
        const origin = descriptor.value;
        if (typeof origin !== 'function') {
            return;
        }
        descriptor.value = async function (...args) {
            let now = +new Date();
            const result = await origin.apply(this, args);
            exports.logger.info(message, '| cost：', +new Date() - now, 'ms');
            return result;
        };
        return descriptor;
    };
}
exports.default = log;
